@@ -27,15 +27,8 @@
 
 #include "rsvg-private.h"
 #include "rsvg-styles.h"
-#include "rsvg-shapes.h"
-#include "rsvg-filter.h"
 #include "rsvg-text.h"
 #include "rsvg-css.h"
-#include "rsvg-mask.h"
-
-#include <libart_lgpl/art_affine.h>
-#include <libart_lgpl/art_svp_ops.h>
-#include <libart_lgpl/art_render_mask.h>
 
 #include "rsvg-shapes.h"
 
@@ -162,19 +155,24 @@ rsvg_tspan_new()
 static void
 rsvg_tchunk_remove_leading(RsvgTChunk * self);
 
+static void
+rsvg_tspan_remove_trailing(RsvgTspan * self);
+
 static void
 rsvg_tspan_remove_leading(RsvgTspan * self)
 {
 	if (!self)
 		return;
-	if (!self->contents->len == 0)
-		return
+	if (self->contents->len == 0)
+		return;
 	rsvg_tchunk_remove_leading(g_ptr_array_index(self->contents, 0));
 }
 
 static void
 rsvg_tchunk_remove_leading(RsvgTChunk * self)
 {
+	if (!self)
+		return;
 	if (self->string)
 		if (self->string->str[0] == ' ')
 			g_string_erase(self->string, 0, 1);
@@ -182,20 +180,6 @@ rsvg_tchunk_remove_leading(RsvgTChunk * self)
 		rsvg_tspan_remove_leading(self->span);
 }
 
-static void
-rsvg_tchunk_remove_trailing(RsvgTChunk * self);
-
-static void
-rsvg_tspan_remove_trailing(RsvgTspan * self)
-{
-	if (!self)
-		return;
-	if (!self->contents->len == 0)
-		return
-	rsvg_tchunk_remove_trailing(g_ptr_array_index(self->contents, 
-												  self->contents->len - 1));
-}
-
 static void
 rsvg_tchunk_remove_trailing(RsvgTChunk * self)
 {
@@ -206,6 +190,17 @@ rsvg_tchunk_remove_trailing(RsvgTChunk * self)
 		rsvg_tspan_remove_trailing(self->span);
 }
 
+static void
+rsvg_tspan_remove_trailing(RsvgTspan * self)
+{
+	if (!self)
+		return;
+	if (self->contents->len == 0)
+		return;
+	rsvg_tchunk_remove_trailing(g_ptr_array_index(self->contents, 
+												  self->contents->len - 1));
+}
+
 typedef struct _RsvgSaxHandlerText {
 	RsvgSaxHandler super;
 	RsvgSaxHandler *parent;
@@ -213,7 +208,7 @@ typedef struct _RsvgSaxHandlerText {
 	GString * id;
 	RsvgTspan * tspan;
 	RsvgTspan * innerspan;
-	RsvgDefsDrawableText * block;
+	RsvgNodeText * block;
 } RsvgSaxHandlerText;
 
 static void
@@ -223,8 +218,10 @@ rsvg_text_handler_free (RsvgSaxHandler *self)
 	z = (RsvgSaxHandlerText *)self;
 
 	/*maybe this isn't the best place for this*/
-	rsvg_tspan_remove_leading(z->tspan);
-	rsvg_tspan_remove_trailing(z->tspan);
+	if(z->tspan != NULL) {
+		rsvg_tspan_remove_leading(z->tspan);
+		rsvg_tspan_remove_trailing(z->tspan);
+	}
 
 	g_string_free(z->id, TRUE);
 	g_free (self);
@@ -377,34 +374,27 @@ rsvg_text_handler_end (RsvgSaxHandler *self, const xmlChar *name)
 }
 
 static void 
-rsvg_defs_drawable_text_free (RsvgDefVal *self)
+rsvg_node_text_free (RsvgNode *self)
 {
-	RsvgDefsDrawableText *z = (RsvgDefsDrawableText *)self;
+	RsvgNodeText *z = (RsvgNodeText *)self;
 	rsvg_tspan_free (z->chunk);
 	g_free (z);
 }
 
 void 
-rsvg_text_render_text (DrawingCtx *ctx,
+rsvg_text_render_text (RsvgDrawingCtx *ctx,
 					   RsvgTspan  *tspan,
 					   const char *text,
 					   gdouble *x,
 					   gdouble *y);
 
-ArtSVP *
-rsvg_text_render_text_as_svp (DrawingCtx *ctx,
-							  RsvgTspan  *tspan,
-							  const char *text,
-							  gdouble *x,
-							  gdouble *y);
-
 static gdouble
-rsvg_text_width  (DrawingCtx *ctx,
+rsvg_text_width  (RsvgDrawingCtx *ctx,
 				  RsvgTspan  *tspan,
 				  const char *text);
 
 static gdouble
-rsvg_text_tspan_width (DrawingCtx *ctx,
+rsvg_text_tspan_width (RsvgDrawingCtx *ctx,
 					   RsvgTspan  *tspan)
 {
 	RsvgTspan  *currentspan = tspan;
@@ -439,10 +429,10 @@ rsvg_text_tspan_width (DrawingCtx *ctx,
 }
 
 static void
-rsvg_tspan_draw(RsvgTspan * self, DrawingCtx *ctx, gdouble *x, gdouble *y, int dominate);
+rsvg_tspan_draw(RsvgTspan * self, RsvgDrawingCtx *ctx, gdouble *x, gdouble *y, int dominate);
 
 static void
-rsvg_tchunk_draw(RsvgTChunk * self, DrawingCtx *ctx, RsvgTspan *span, gdouble *x, gdouble *y)
+rsvg_tchunk_draw(RsvgTChunk * self, RsvgDrawingCtx *ctx, RsvgTspan *span, gdouble *x, gdouble *y)
 {
 	if (self->string)
 		rsvg_text_render_text (ctx, span, self->string->str, x, y);
@@ -455,7 +445,7 @@ rsvg_tchunk_draw(RsvgTChunk * self, DrawingCtx *ctx, RsvgTspan *span, gdouble *x
 }
 
 static void
-rsvg_tspan_draw(RsvgTspan * self, DrawingCtx *ctx, gdouble *x, gdouble *y, int dominate)
+rsvg_tspan_draw(RsvgTspan * self, RsvgDrawingCtx *ctx, gdouble *x, gdouble *y, int dominate)
 {
 	unsigned int i;
 	rsvg_state_reinherit_top(ctx, &self->state, dominate);
@@ -492,94 +482,23 @@ rsvg_tspan_draw(RsvgTspan * self, DrawingCtx *ctx, gdouble *x, gdouble *y, int d
 	}
 }
 
-static ArtSVP *
-rsvg_tspan_draw_as_svp(RsvgTspan * self, DrawingCtx *ctx, gdouble *x, gdouble *y, int dominate);
-
-static ArtSVP *
-rsvg_tchunk_draw_as_svp(RsvgTChunk * self, DrawingCtx *ctx, RsvgTspan *span, gdouble *x, gdouble *y)
-{
-	if (self->string)
-		return rsvg_text_render_text_as_svp (ctx, span, self->string->str, x, y);
-	if (self->span)
-		{
-			ArtSVP * output;
-			rsvg_state_push(ctx);
-			output = rsvg_tspan_draw_as_svp (self->span, ctx, x, y, 0);
-			rsvg_state_pop(ctx);
-			return output;
-		}
-	return NULL;
-}
-
-/*todo: remove unnesicary code duplication*/
-
-static ArtSVP *
-rsvg_tspan_draw_as_svp(RsvgTspan * self, DrawingCtx *ctx, gdouble *x, gdouble *y, int dominate)
-{
-	unsigned int i;
-	ArtSVP *svp1, *svp2, *svp3;
-	svp1 = NULL;
-
-	rsvg_state_reinherit_top(ctx, &self->state, dominate);
-	if (self->hasx || self->hasy)
-		{
-			switch (rsvg_state_current(ctx)->text_anchor)
-				{
-				case TEXT_ANCHOR_START:
-					*x = self->x;		
-					break;
-				case TEXT_ANCHOR_MIDDLE:
-					*x = self->x - rsvg_text_tspan_width (ctx, self) / 2;
-					break;
-				case TEXT_ANCHOR_END:
-					*x = self->x - rsvg_text_tspan_width (ctx, self);
-					break;
-				}
-			*y = self->y;
-		}
-
-	for (i = 0; i < self->contents->len; i++) {
-		svp2 = rsvg_tchunk_draw_as_svp (g_ptr_array_index(self->contents, i), ctx, self, x, y);
-		if (svp1 != NULL)
-			{
-				svp3 = art_svp_union(svp2, svp1);
-				art_free(svp1);
-				svp1 = svp3;
-			}
-		else
-			svp1 = svp2;
-	}
-	return svp1;
-}
-
-
 static void 
-rsvg_defs_drawable_text_draw (RsvgDefsDrawable * self, DrawingCtx *ctx, 
+rsvg_node_text_draw (RsvgNode * self, RsvgDrawingCtx *ctx, 
 							  int dominate)
 {
 	gdouble x, y;
-	RsvgDefsDrawableText *text = (RsvgDefsDrawableText*)self;
+	RsvgNodeText *text = (RsvgNodeText*)self;
 
 	rsvg_tspan_draw(text->chunk, ctx, &x, &y, dominate);
 }
 
-static ArtSVP *
-rsvg_defs_drawable_text_draw_as_svp (RsvgDefsDrawable * self, DrawingCtx *ctx, 
-									 int dominate)
-{
-	gdouble x, y;
-	RsvgDefsDrawableText *text = (RsvgDefsDrawableText*)self;
-
-	return rsvg_tspan_draw_as_svp(text->chunk, ctx, &x, &y, dominate);
-}
-
 void
 rsvg_start_text (RsvgHandle *ctx, RsvgPropertyBag *atts)
 {
 	double x, y, dx, dy, font_size;
 	const char * klazz = NULL, * id = NULL, *value;
 	RsvgState state;
-	RsvgDefsDrawableText *text;
+	RsvgNodeText *text;
 	RsvgSaxHandlerText *handler = g_new0 (RsvgSaxHandlerText, 1);
 
 	handler->super.free = rsvg_text_handler_free;
@@ -610,18 +529,16 @@ rsvg_start_text (RsvgHandle *ctx, RsvgPropertyBag *atts)
 			rsvg_parse_style_attrs (ctx, &state, "text", klazz, id, atts);
 		}
 	
-	text = g_new (RsvgDefsDrawableText, 1);
+	text = g_new (RsvgNodeText, 1);
 
-	text->super.super.type = RSVG_DEF_PATH;
-	text->super.super.free = rsvg_defs_drawable_text_free;
-	text->super.draw = rsvg_defs_drawable_text_draw;
-	text->super.draw_as_svp = rsvg_defs_drawable_text_draw_as_svp;
-	rsvg_defs_set (ctx->defs, id, &text->super.super);
+	text->super.type = RSVG_NODE_PATH;
+	text->super.free = rsvg_node_text_free;
+	text->super.draw = rsvg_node_text_draw;
+	rsvg_defs_set (ctx->defs, id, &text->super);
 	
-	text->super.parent = (RsvgDefsDrawable *)ctx->current_defs_group;
+	text->super.parent = (RsvgNode *)ctx->currentnode;
 	if (text->super.parent != NULL)
-		rsvg_defs_drawable_group_pack((RsvgDefsDrawableGroup *)text->super.parent, 
-									  &text->super);
+		rsvg_node_group_pack(text->super.parent, &text->super);
 
 	handler->id = g_string_new(id);
 
@@ -649,7 +566,7 @@ typedef struct _RsvgTextLayout RsvgTextLayout;
 struct _RsvgTextLayout
 {
 	PangoLayout * layout;
-	DrawingCtx  * ctx;
+	RsvgDrawingCtx  * ctx;
 	TextAnchor anchor;
 	RsvgTspan    *span;
 	gdouble x, y;
@@ -714,7 +631,7 @@ rsvg_text_ft2_subst_func (FcPattern *pattern,
 }
 
 static PangoContext *
-rsvg_text_get_pango_context (DrawingCtx *ctx)
+rsvg_text_get_pango_context (RsvgDrawingCtx *ctx)
 {
 	PangoContext    *context;
 	PangoFT2FontMap *fontmap;
@@ -742,7 +659,7 @@ rsvg_text_layout_free(RsvgTextLayout * layout)
 }
 
 static RsvgTextLayout *
-rsvg_text_layout_new (DrawingCtx *ctx,
+rsvg_text_layout_new (RsvgDrawingCtx *ctx,
 					  RsvgState *state,
 					  const char *text)
 {
@@ -1097,7 +1014,7 @@ rsvg_text_layout_render (RsvgTextLayout     *layout,
 }
 
 static GString * 
-rsvg_text_render_text_as_string (DrawingCtx *ctx,
+rsvg_text_render_text_as_string (RsvgDrawingCtx *ctx,
 								 RsvgTspan  *tspan,
 								 const char *text,
 								 gdouble *x,
@@ -1136,7 +1053,7 @@ rsvg_text_render_text_as_string (DrawingCtx *ctx,
 }
 
 void
-rsvg_text_render_text (DrawingCtx *ctx,
+rsvg_text_render_text (RsvgDrawingCtx *ctx,
 					   RsvgTspan  *tspan,
 					   const char *text,
 					   gdouble *x,
@@ -1148,21 +1065,6 @@ rsvg_text_render_text (DrawingCtx *ctx,
 	g_string_free(render, TRUE);
 }
 
-ArtSVP *
-rsvg_text_render_text_as_svp (DrawingCtx *ctx,
-							  RsvgTspan  *tspan,
-							  const char *text,
-							  gdouble *x,
-							  gdouble *y)
-{
-	GString * render;
-	ArtSVP * output;
-	render = rsvg_text_render_text_as_string (ctx,tspan,text,x,y);
-	output = rsvg_render_path_as_svp (ctx, render->str);
-	g_string_free(render, TRUE);
-	return output;
-}
-
 static gdouble
 rsvg_text_layout_width  (RsvgTextLayout *layout)
 {
@@ -1190,7 +1092,7 @@ rsvg_text_layout_width  (RsvgTextLayout *layout)
 }
 
 static gdouble
-rsvg_text_width       (DrawingCtx *ctx,
+rsvg_text_width       (RsvgDrawingCtx *ctx,
 					   RsvgTspan  *tspan,
 					   const char *text)
 {
